/** \file   my_hash.h
 *  \brief  A simple hash table
 *  \author Takahisa Toda
 */
#ifndef MY_HASH_H
#define MY_HASH_H

#define HASH_DEFAULT_SIZE        (UINTMAX_C(1) << 24)

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/** \brief a linked list for a hash table*/
struct hash_entry {
  uintptr_t key;
  uintptr_t data;
  struct hash_entry *next;
};

/** \brief hash table (chaining)*/
typedef struct {
  uintmax_t nbucket;          //!< the number of buckets
  struct hash_entry **bucket; //!< the i-th bucket: bucket[i], where 0 <= i < nbucket.
} my_hash;


/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

extern my_hash *ht_create(uintmax_t nbuckets);
extern void ht_destroy(my_hash *h);
extern void ht_add(uintptr_t key, uintptr_t data, my_hash *h);
extern int  ht_search(uintptr_t key, uintptr_t *pdata, const my_hash *h);
extern uintmax_t ht_maxdepth(const my_hash *h);
extern void ht_enlarge(my_hash *h);

#endif /*MY_HASH_H*/
